/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with {library}; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package ch.hortis.sonar.mvn;

import java.util.Date;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.FlushModeType;
import javax.persistence.NoResultException;

import org.apache.maven.plugin.MojoExecutionException;

import ch.hortis.sonar.model.Collectable;
import ch.hortis.sonar.model.MavenProject;
import ch.hortis.sonar.model.MetricMeasure;
import ch.hortis.sonar.model.Snapshot;
import ch.hortis.sonar.model.WithFile;
import ch.hortis.sonar.mvn.mc.FilesRepository;
import ch.hortis.sonar.mvn.mc.FilesRepositoryImpl;
import ch.hortis.sonar.mvn.mc.MeasuresCollector;
import ch.hortis.sonar.service.MavenProjectService;
import ch.hortis.sonar.service.MetricService;
import ch.hortis.sonar.service.RulesService;

/**
 * Main sonar plugin reports for plugins metrics parsing and historic generation
 * @goal collect
 */
public class CollectMojo extends CoreMojo {
  
  private FilesRepository filesRepository;

  public void setFilesRepository(FilesRepository filesRepository) {
    this.filesRepository = filesRepository;
  }
  
  protected List<Report> getTargetReports() {
    if (isLight()){
     return Report.getLightReports(); 
    }
    return Report.getReports(); 
  }

  public void execute() throws MojoExecutionException {
    try {
      MavenProjectService service = new MavenProjectService(getEntityManager());
      MavenProject sonarProject;
      try {
        sonarProject = service.getMavenProject(getProject().getGroupId(), getProject().getArtifactId(), getBranch());
      } catch ( NoResultException ex ) {
        throw new MojoExecutionException("Unable to find project " + getProject().getGroupId() + ":" + 
            getProject().getArtifactId() + ":" + getBranch() + " in database");
      }
      collect(sonarProject);
    } finally {
      destroyManager();
    }
  }

  private void collectAndPersist(Snapshot snapshot) throws MojoExecutionException {
    if (!getProject().getPackaging().equals("pom")) {
      if (filesRepository == null) {
        filesRepository = new FilesRepositoryImpl(getLog());
      }
      MetricService metricService = new MetricService(getEntityManager());
      RulesService rulesService = new RulesService(getEntityManager());
      for (Report report : getTargetReports()) {
        MeasuresCollector collector = report.getMeasuresCollector();
        if (collector.initialize(getProject(), metricService, rulesService, filesRepository)) {
          List<Collectable> collected = collector.collect();
          persistCollectables(snapshot, collected);
        }
      }
    }
  }

  private void collect(MavenProject sonarProject) throws MojoExecutionException {
    Snapshot snapshot = new Snapshot();
    snapshot.setCreatedAt(new Date());
    snapshot.setMavenProject(sonarProject);
    snapshot.setVersion(getProject().getVersion());
    getEntityManager().setFlushMode(FlushModeType.COMMIT);
    getEntityManager().getTransaction().begin();
    getEntityManager().persist(snapshot);
    collectAndPersist(snapshot);
    setSnapshotId(snapshot.getId(), getProject());
    getEntityManager().getTransaction().commit();
  }
  
  private void persistCollectables(Snapshot snapshot, List<Collectable> collectables) throws MojoExecutionException {
    EntityManager manager = getEntityManager();
    for (Collectable collectable : collectables) {
      if (collectable instanceof WithFile) {
        if (((WithFile) collectable).getFile() == null) {
          getLog().warn("FileMeasure without file : " + collectable + ", snapshot=" + snapshot.getId());
          continue;
        } else {
          ((WithFile) collectable).getFile().setSnapshot(snapshot);
        }
      }
      if (collectable instanceof MetricMeasure && ((MetricMeasure) collectable).getValue().isNaN()) {
        getLog().warn("Measure with NaN value : " + collectable + ", snapshot=" + snapshot.getId());
        continue;
      }
      collectable.setId(null);
      collectable.setSnapshot(snapshot);
      manager.persist(collectable);
    }
  }

}