/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with {library}; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package ch.hortis.sonar.mvn;

import javax.persistence.EntityManager;
import javax.persistence.NoResultException;

import org.apache.maven.model.CiManagement;
import org.apache.maven.model.IssueManagement;
import org.apache.maven.model.Scm;
import org.apache.maven.plugin.MojoExecutionException;

import ch.hortis.sonar.model.MavenProject;
import ch.hortis.sonar.model.ProjectLink;
import ch.hortis.sonar.service.MavenProjectService;

/**
 * This plugin will create the project hierarchy in the database and create the
 * snapshot group to use for measures collection
 *
 * @goal project
 * @aggregator
 */
public class ProjectMojo extends CoreMojo {

  public void execute() throws MojoExecutionException {
    EntityManager manager = getEntityManager();
    try {
      saveProjects(manager);
    } finally {
      if ( manager.getTransaction().isActive() ) {
        manager.getTransaction().rollback();
      }
      destroyManager();
    }
  }

  private void saveProjects(EntityManager manager) throws MojoExecutionException {
    manager.getTransaction().begin();
    for (org.apache.maven.project.MavenProject reactorProject : getReactorProjects()) {
      saveProject(reactorProject);
    }
    manager.getTransaction().commit();
  }
 
  private void saveProject(org.apache.maven.project.MavenProject mavenProject) throws MojoExecutionException {
    MavenProjectService service = new MavenProjectService(getEntityManager());
    MavenProject sonarProject;
    try {
      sonarProject = service.getMavenProject(mavenProject.getGroupId(), mavenProject.getArtifactId(), getBranch());
      updateProject(sonarProject, mavenProject);
      getEntityManager().merge(sonarProject);
    } catch (NoResultException e) {
      sonarProject = new MavenProject();
      sonarProject.setArtifactId(mavenProject.getArtifactId());
      sonarProject.setGroupId(mavenProject.getGroupId());
      sonarProject.setBranch(getBranch());
      sonarProject.setEnabled(true);
      updateProject(sonarProject, mavenProject);
      getEntityManager().persist(sonarProject);
    }
  }

  private void updateProject(MavenProject sonarProject, org.apache.maven.project.MavenProject mavenProject) {
    sonarProject.setName(mavenProject.getName());
    sonarProject.setDescription(mavenProject.getDescription());
    updateProjectLink(ProjectLink.LINK_HOME_PAGE, mavenProject.getUrl(), sonarProject);
    Scm scmConfig = mavenProject.getScm();
    if (scmConfig == null) {
      scmConfig = new Scm();
    }
    updateProjectLink(ProjectLink.LINK_SCM_URL, scmConfig.getUrl(), sonarProject);
    updateProjectLink(ProjectLink.LINK_SCM_DEV_CONNECTION, scmConfig.getDeveloperConnection(), sonarProject);
    updateProjectLink(ProjectLink.LINK_SCM_RO_CONNECTION, scmConfig.getConnection(), sonarProject);

    CiManagement cimConfig = mavenProject.getCiManagement();
    if (cimConfig == null) {
      cimConfig = new CiManagement();
    }
    updateProjectLink(ProjectLink.LINK_CONTINUOUS_INTEGRATION, cimConfig.getUrl(), sonarProject);

    IssueManagement imConfig = mavenProject.getIssueManagement();
    if (imConfig == null) {
      imConfig = new IssueManagement();
    }
    updateProjectLink(ProjectLink.LINK_ISSUES_TRACKER, imConfig.getUrl(), sonarProject);
  }

  private void updateProjectLink(String linkType, String href, MavenProject sonarProject) {
    if (href != null && !"".equals(href)) {
      ProjectLink link = sonarProject.getProjectLinkByType(linkType);
      if (link == null) {
        link = new ProjectLink();
        link.setMavenProject(sonarProject);
        link.setType(linkType);
        sonarProject.getProjectLinks().add(link);
      }
      link.setHref(href);
    }
  }

}