/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with {library}; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package ch.hortis.sonar.mvn.mc;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.project.MavenProject;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import ch.hortis.sonar.model.Collectable;
import ch.hortis.sonar.model.File;
import ch.hortis.sonar.model.Rule;
import ch.hortis.sonar.model.RuleFailure;
import ch.hortis.sonar.model.RuleFailureLevel;

public class PMDCollector extends BaseMeasuresCollector {
  private java.io.File pmdFile = null;
  private XmlReportParser parser = null;


  public boolean initialize(MavenProject project) {
    pmdFile = findFileFromBuildDirectory(project, "pmd.xml");
    boolean ok = false;
    if (pmdFile != null) {
      parser = new XmlReportParser();
      parser.parse(pmdFile);
      ok = true;
    }
    return ok;
  }

  public List<Collectable> collect() throws MojoExecutionException {
    return collectRuleFailures();
  }

  public List<Collectable> collectRuleFailures() throws MojoExecutionException {
    ArrayList<Collectable> collectables = new ArrayList<Collectable>();
    if (pmdFile != null) {
      NodeList files = parser.executeXPathNodeList("/pmd/file");
      for (int i = 0; i < files.getLength(); i++) {
        Element element = (Element) files.item(i);
        String name = element.getAttribute("name");
        String filename = StringUtils.substringAfterLast(name, "/");
        if (filename == null || "".equals(filename)) {
          filename = StringUtils.substringAfterLast(name, "\\");
        }
        List<Element> violations = parser.getChildElements(element, "violation");
        for (Element violation : violations) {
          Rule rule = getRuleService().getRuleByPluginKey(violation.getAttribute("rule"));
          if (rule == null) {
            continue;
          }
          RuleFailure failure = new RuleFailure();
          collectables.add(failure);
          failure.setRule(rule);

          File file = getFilesRepository().getFile(violation.getAttribute("package"), filename);
          failure.setFile(file);
          String line = violation.getAttribute("beginline");
          if (line != null && !"".equals(line)) {
            failure.addParameter("line", Double.valueOf(line));
          }
          failure.setMessage(violation.getFirstChild().getNodeValue());
          int priority = Integer.parseInt(violation.getAttribute("priority"));
          if (priority <= 2) {
            failure.setLevel(RuleFailureLevel.ERROR);
          } else if (priority == 3 || priority == 4) {
            failure.setLevel(RuleFailureLevel.WARNING);
          } else {
            failure.setLevel(RuleFailureLevel.INFO);
          }
        }
      }
    }
    return collectables;
  }

}