/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with {library}; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package ch.hortis.sonar.mvn.reports;

import java.io.File;
import java.io.IOException;
import java.util.Arrays;
import java.util.List;

import org.apache.maven.model.Plugin;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.logging.Log;
import org.apache.maven.project.MavenProject;

import ch.hortis.sonar.mvn.MavenContext;
import ch.hortis.sonar.mvn.PomUtils;

public class CheckstyleHandler extends ReportHandler {

  public void configurePlugin(MavenProject pom, Plugin plugin) throws MojoExecutionException {
    unsetConfigParameter( plugin, "outputDirectory" );
    unsetConfigParameter( plugin, "outputFile" );
    setConfigParameter( plugin, "outputFileFormat", "xml" );
    setConfigParameter( plugin, "consoleOutput", "false" );
    setConfigParameter( plugin, "enableRSS", "false" );
    setConfigParameter( plugin, "failsOnError", "false" );
    setConfigParameter( plugin, "linkXRef", "false" );
    
    String ckeckStylePath = pom.getBuild().getDirectory() + "/sonar/checkstyle.xml";
    setConfigParameter( plugin, "configLocation", ckeckStylePath );
    PomUtils.copyPluginDependencies(pom, plugin);
  }

  public boolean execute( MavenContext mavenContext, Log log ) throws MojoExecutionException {
    String ckeckStylePath = mavenContext.getExecutedProject().getBuild().getDirectory() + "/sonar/checkstyle.xml";
    File checkstyle = new File( ckeckStylePath );
    try {
    	mavenContext.getWebInterfaceService().getRemoteCheckstyleConfiguration(checkstyle);
    } catch ( IOException e ) {
    	throw new MojoExecutionException( "Error during sonar checkstyle config file download", e );
    }
    return true;
  }

  public List<String> getGoals() {
    return Arrays.asList( new String[]{"checkstyle"} );
  }

  public boolean mustFailOnError() {
    return true;
  }
}
