/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with {library}; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package ch.hortis.sonar.mvn.reports;

import java.io.File;
import java.util.Arrays;
import java.util.List;

import org.apache.maven.model.Plugin;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.logging.Log;
import org.apache.maven.project.MavenProject;
import org.codehaus.plexus.util.xml.Xpp3Dom;

import ch.hortis.sonar.mvn.MavenContext;
import ch.hortis.sonar.mvn.PomUtils;

public class PMDHandler extends ReportHandler {

  public boolean executeEvenIfNoJavaSources() {
    return false;
  }
  
  public void configurePlugin(MavenProject pom, Plugin plugin) {
    unsetConfigParameter(plugin, "outputDirectory");
    unsetConfigParameter(plugin, "targetDirectory");
    setConfigParameter(plugin, "format", "xml");
    setConfigParameter(plugin, "linkXRef", "false");
    String javaVersion = PomUtils.getJavaVersion(pom);
    if (javaVersion != null) {
      if (javaVersion.equals("1.1") || javaVersion.equals("1.2") ){
        setConfigParameter(plugin, "targetJdk", "1.3");
      } else {
        setConfigParameter(plugin, "targetJdk", javaVersion);
      }
    }
    unsetConfigParameter(plugin, "rulesets");
    
    Xpp3Dom conf = (Xpp3Dom) plugin.getConfiguration();
    Xpp3Dom rulesets = new Xpp3Dom("rulesets");
    Xpp3Dom ruleset = new Xpp3Dom("ruleset");
    
    ruleset.setValue(pom.getBuild().getDirectory() + "/sonar/pmd.xml");
    rulesets.addChild( ruleset );
    conf.addChild(rulesets);
    PomUtils.copyPluginDependencies(pom, plugin);
  }
  
  public boolean execute( MavenContext mavenContext, Log log ) throws MojoExecutionException {
    File pmd = new File( mavenContext.getExecutedProject().getBuild().getDirectory() + "/sonar/pmd.xml" );
    try {
    	mavenContext.getWebInterfaceService().getRemotePMDConfiguration(pmd);
    } catch ( Exception e ) {
      throw new MojoExecutionException( "Error during sonar PMD config file download", e );
    }
    return true;
  }

  public List<String> getGoals() {
    return Arrays.asList(new String[]{"pmd"});
  }

  public boolean mustFailOnError() {
    return true;
  }
}
